/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.evasion.plugin.geoloc.dataimport.gis;

/**
 *
 * @author glon-56610
 */
public final class GISFormat {

    private GISFormat() {
    }

    /**
     * <b>Region Font Code.</b>  A code that determines the character mapping used in fields containing feature name information (see Character Set Conversion Table:<br>
     * 1 = Americas/Western Europe;<br>
     * 2 = Eastern Europe;<br>
     * 3 = Africa/Middle East;<br>
     * 4 = Russia/ Central Asia;<br>
     * 5 = Asia/Pacific;<br>
     * 6 = Vietnam.<br>
     */
    public static final int RC = 0;

    /**
     * <b>Unique Feature Identifier.</b>  A number which uniquely identifies a feature.
     */
    public static final int UFI = 1;

    /**
     * <b>Unique Name Identifier.</b>  A number which uniquely identifies a name.
     */
    public static final int UNI = 2;

    /**
     * <b>Latitude</b> of the feature in ± decimal degrees; DD; (± dd.dd...):<br>
     * no sign (+) = North;<br>
     * negative sign (-) = South<br>
     */
    public static final int LAT = 3;

    /**
     * <b>Longitude</b> of the feature in ± decimal degrees; DD; (± dd.dd...):<br>
     * no sign (+) = East;<br>
     * negative sign (-) = West.<br>
     */
    public static final int LONG = 4;

    /**
     * 	<b>Latitude</b> of the feature in ±degrees, minutes, and seconds; DMS; (±ddmmss):<br>
     * no sign (+) = North;<br>
     * negative sign (-) = South.<br>
     */
    public static final int DMS_LAT = 5;

    /**
     * <b>Longitude</b> of the feature in ±degrees, minutes, and seconds; DMS; (±dddmmss) :<br>
     * no sign (+) = East;<br>
     * negative sign (-) = West.<br>
     */
    public static final int DMS_LONG = 6;

    /**
     * <b>Military Grid Reference System coordinates.</b><br>
     * MGRS is an alpha-numeric system for expressing UTM/UPS coordinates. A single alpha-numeric value references an area that is unique for the entire earth. 
     */
    public static final int MGRS = 7;

    /**
     * <b>Joint Operations Graphic</b> reference.
     */
    public static final int JOG = 8;

    /**
     * <b>Feature Classification:</b> Nine (9) major Geoname feature categories into which similar feature designations are grouped.<br>
     * A = Administrative region type feature<br>
     * P = Populated place type feature<br>
     * V = Vegetation type feature<br>
     * L = Locality or area type feature<br>
     * U = Undersea type feature<br>
     * R = Streets, highways, roads, or railroad type feature<br>
     * T = Hypsographic type feature<br>
     * H = Hydrographic type feature<br>
     * S = Spot type feature<br>
     */
    public static final int FC = 9;

    /**
     * <b>Feature Designation Code.</b>  A two to five-character code used to identify the type of Geoname feature a name is applied to.
     */
    public static final int DSG = 10;

    /**
     * 	<b>Populated Place Classification.</b>  A graduated numerical scale denoting the relative importance of a populated place.  The scale ranges from 1,  relatively high, to 5, relatively low.  The scale could also include NULL (no value) as a value for populated places with unknown or undetermined classification.
     */
    public static final int PC = 11;

    /**
     * <b>Primary Country Code.</b> A two alphabetic character code from the Geopolitical Codes (formerly FIPS 10-4 Standard) that uniquely identify a geopolitical entity (countries, dependencies, and areas of special sovereignty).
     */
    public static final int CC1 = 12;

    /**
     * <b>First-order administrative division code.</b> A two alpha-numeric code from the Geopolitical Codes (formerly FIPS 10-4 Standard) describing a primary administrative division of a geopolitical entity, such as a state in the United States.
     */
    public static final int ADM1 = 13;

    /**
     * Population figures.
     */
    public static final int POP = 14;

    /**
     * 	Elevation in meters.
     */
    public static final int ELEV = 15;

    /**
     * <b>Secondary Country Code.</b> A two alphabetic character code from the Geopolitical Codes (formerly FIPS 10-4 Standard) uniquely identifying the country code of a particular Geoname feature name if different than that of the primary country code of that Geoname feature (International features). 
     */
    public static final int CC2 = 16;

    /**
     * <b>Name Type:</b><br>
     * C = Conventional name<br>
     * N = BGN Standard - Official local name for a feature<br>
     * NS = Native Script Name<br>
     * D = Unverified or daggered name<br>
     * DS = Unverified name in native script<br>
     * H = Historic name<br>
     * HS = Historic name in native script<br>
     * P = Provisional name<br>
     * PS = Provisional name in native script<br>
     * V = Variant or alternate name<br>
     * VS = Variant name in native script <br>
     */
    public static final int NT = 17;

    /**
     * <b>Language Code.</b> A three alphabetic character code (ISO 639-3) uniquely identifying the language assigned to a feature name.
     */
    public static final int LC = 18;

    /**
     * A part of the full name that could substitute for the full name (Unicode UTF-8).
     */
    public static final int SHORT_FORM = 19;

    /**
     * The descriptive part of the full name such as Mount, River, or Sea (Unicode UTF-8 - generally does not apply to populated place names). 	
     */
    public static final int GENERIC = 20;

    /**
     * <b>Sort name - reading order.</b> A form of the full name that allows for alphabetical sorting of the file into gazetteer sequence. All character/diacritic combinations and special characters are substituted with QWERTY (visible U.S. English keyboard) characters, all characters are upper-cased, numerals are converted to lower-case characters (0-9 = a-j), spaces are stripped out, and commas replaced with space.  The substitution is done according to a character mapping table that is Region Font Code specific (see Character Set Conversion Table.  This field is included for the benefit of the end user of the data to aid in the sorting of names if required.
     */
    public static final int SORT_NAME_RO = 21;

    /**
     * 	<b>Full name - reading order.</b> The full name is the complete name that identifies a named feature.  The full name is output in reading order, "Mount Everest", vs. reversed generic, "Everest, Mount", as stored in the database.
     */
    public static final int FULL_NAME_RO = 22;

    /**
     * 	<b>Full name - reading order with no diacritics.</b> Same as the full name but the character/diacritic combinations and special characters are substituted with QWERTY (visible U.S. English keyboard) characters while still maintaining casing and spaces. The substitution is done according to a character mapping table that is Region Font Code specific (see Character Set Conversion Table).
     */
    public static final int FULL_NAME_ND_RO = 23;

    /**
     * <b>Sort name - reversed generic.</b> A form of the full name that allows for alphabetical sorting of the file into gazetteer sequence. All character/ diacritic combinations and special characters are substituted with QWERTY (visible U.S. English keyboard) characters, all characters are upper-cased, numerals are converted to lower-case characters (0-9 = a-j), spaces are stripped out, and commas replaced with space. The substitution is done according to a character mapping table that is Region Font Code specific (see Character Set Conversion Table). This field is included for the benefit of the end user of the data to aid in the sorting of names if required.
     */
    public static final int SORT_NAME_RG = 24;

    /**
     * <b>Full name - reversed generic.</b> The full name is the complete name that identifies a named feature. The full name is output in reversed generic, "Everest, Mount," as stored in the database, vs. reading order, "Mount Everest."
     */
    public static final int FULL_NAME_RG = 25;

    /**
     * <b>Full name - reversed generic with no diacritics.</b> Same as the full name but the character/diacritic combinations and special characters are substituted with QWERTY (visible U.S. English keyboard) characters while still maintaining casing and spaces. The substitution is done according to a character mapping table that is Region Font Code Specific (see Character Set Conversion Table).
     */
    public static final int FULL_NAME_ND_RG = 26;

    /**
     * This field holds a geopolitical policy note concerning the feature, populated when CC1 and ADM1 are NULL, and it could contain multiple notes (may contain data in Unicode UTF-8). If multiple notes are present, they will be delimited by semicolon followed by four spaces (1:note1;    2:note2;    3:note3;    etc...).
     */
    public static final int NOTE = 27;

    /**
     * The date a new feature was added or any part of an existing feature was modified (YYYY-MM-DD). 
     */
    public static final int MODIFY_DATE = 28;
    
    /**
     * Max Column.
     * 
     */
    public static final int SIZE_COLUMN_FILE = 29;
}
